#include <stdio.h>
#include <sys/types.h>
#include "util.h"



int main() {



    //load parameters from file
    phys_para const pp = load_para();

    
    // ------------ parameters ---------------
   
    //boundary conditions along y and x if these are set to 0.0 then the system has open boundaries, set to 1.0 means periodic
    double const beta=pp.t; //y-direction
    double const alpha=pp.V;//x-direction
    
    // imaginary part of frequency
    double const eta=pp.es;
    
    //system size
    int const L=pp.L;
    
    //spring constants on x and y
    float Kx=pp.exy;
    float Ky=pp.exz;

    // magnitude of magnetic field
    float Bz=pp.eyz;


    //charge and mass
    float q =1.0;
    float m=1.0;
   
    //real part of omega, and the variable which will contain om+i\eta
    float omega=0.0;
    gsl_complex om;
    
    // ------------end parameters ---------------

//------------------------- print parameters--------------------------------
    printf("-------------------------- Parameters ------------------------------\n");
    printf("\tL=%d\t beta=%g \t alpha=%g \t eta=%g\t Kx=%g\t Ky=%g\t B=%g\n", L, beta,alpha,eta,Kx,Ky,Bz);
    printf("--------------------------------------------------------------------\n\n");
    //--------------------------------------------------------------------------

    
    
    //file handle
    FILE * f;

    //loop indices
    int i, j,j2;



   //Matrices needed to construct the Hamiltonian blocks
    gsl_matrix_complex * A = gsl_matrix_complex_alloc (2,2);
    gsl_matrix_complex * Bx = gsl_matrix_complex_alloc (2,2);
    gsl_matrix_complex * By = gsl_matrix_complex_alloc (2,2);
    
    
    //Hamiltonian blocks along a single 1D chain
    gsl_matrix_complex * Abig = gsl_matrix_complex_alloc (2*(L),2*(L));
    gsl_matrix_complex * Bbig = gsl_matrix_complex_alloc (2*(L),2*(L));

    //full 2D Hamiltonian
    gsl_matrix_complex * H = gsl_matrix_complex_alloc (2*(L)*(L),2*(L)*(L));

    //Green's function
    gsl_matrix_complex * G = gsl_matrix_complex_alloc (2*(L)*(L),2*(L)*(L));

    //utility object needed to invert H to get the Green's function
    gsl_permutation * p = gsl_permutation_alloc (2*(L)*(L));

    //Matrix that stores the position space spectral function
    gsl_matrix *spec = gsl_matrix_alloc(L,L);

    //Number that holds the full spectral weight of the system at a particular frequency
    float sfull=0.0;

    //utility complex matrix and complex numbers needed for calculating
    gsl_matrix_complex * cmat_temp = gsl_matrix_complex_alloc (2,2);
    gsl_complex A_temp, B_temp,By_temp,var;



//////////////////////////////Begin Preprocessing/////////////////
    
    gsl_matrix_complex_set_zero(A);
    gsl_matrix_complex_set_zero(Bx);
    gsl_matrix_complex_set_zero(By);
    
    gsl_matrix_complex_set_zero(Abig);
    gsl_matrix_complex_set_zero(Bbig);

  
     f = fopen ("sfun.dat", "w");// This is to clear existing content
     fclose(f);
     f = fopen ("sfull.dat", "w");// This is to clear existing content
     fclose(f);
     



  
    ///////////////////////////  Load list of omegas //////////////////
    gsl_matrix * omlist=gsl_matrix_alloc(10,3);

    double omarray[]={0};

    //index of initial and final omegas; initialized here these will change once the omega list has been loaded
    int omi=0;
    int omL=100;

    omlist=load_klistone();//loads the list

    //test to make sure we got a good list   
    if(omlist==NULL)
    {
        printf("Failed to load omega's");
        return -1;
    }

    omL = omlist->size1; // # of lines

    if( omL < 1)
    {
        printf("\n No omegas read from file klist.txt\n");
        return 0;
    }

    omi=0;
    printf("%d omegas loaded.\n", omL);
///////////////////End Preprocessing//////////////////////////


////////////Begin Main Processing Loop///////////////////////////
    printf("\nCalculating spectra...\n\n");
     

    while(omi< omL)
    {

        omarray[0]=(gsl_matrix_get(omlist,omi,0))*PI;
        omega=2*omarray[0];

        omi++;

        printf("omega = %g \n", omarray[0]/PI);




         ///////////////////create Hamiltonian along chains/////////////////////////
            om=gsl_complex_rect(omega,eta);
            var = gsl_complex_mul(om,om);
            var = gsl_complex_mul_real(var,m);
            
            gsl_matrix_complex_set(A,0,0,gsl_complex_add(gsl_complex_rect(-2.0*Kx,0.0),var));
            gsl_matrix_complex_set(A,1,1,gsl_complex_add(gsl_complex_rect(-2.0*Ky,0.0),var));
            var = gsl_complex_mul(gsl_complex_rect(0,-q*Bz),om);
            gsl_matrix_complex_set(A,0,1,var);
            var = gsl_complex_mul(gsl_complex_rect(0,q*Bz),om);
            gsl_matrix_complex_set(A,1,0,var);
            
            gsl_matrix_complex_set(Bx,0,0,gsl_complex_rect(Kx,0));

            gsl_matrix_complex_set(By,1,1,gsl_complex_rect(Ky,0));

       

          for(j=0;j<2;j++)
            for(j2=0;j2<2;j2++)
            {

             for(i=1;i<(L);i++)
             {

             A_temp = gsl_matrix_complex_get(A,j,j2);
             B_temp = gsl_matrix_complex_get(Bx,j,j2);
             By_temp =gsl_matrix_complex_get(By,j,j2);

                gsl_matrix_complex_set(Abig,(i-1)*2+j,(i-1)*2+j2, A_temp);
                gsl_matrix_complex_set(Abig,(i-1)*2+j2,i*2+j, gsl_complex_conjugate(B_temp));
                gsl_matrix_complex_set(Abig,i*2+j,(i-1)*2+j2, B_temp);
                
                gsl_matrix_complex_set(Bbig,(i-1)*2+j,(i-1)*2+j2,By_temp);
            }


               A_temp = gsl_matrix_complex_get(A,j,j2);
               B_temp = gsl_matrix_complex_get(Bx,j,j2);
               By_temp = gsl_matrix_complex_get(By,j,j2);
          
               gsl_matrix_complex_set(Abig,0+j2,(L-1)*2+j, gsl_complex_mul_real(gsl_complex_conjugate(B_temp),alpha));
               gsl_matrix_complex_set(Abig,(L-1)*2+j,0+j2, gsl_complex_mul_real(B_temp,alpha));
               gsl_matrix_complex_set(Abig,(L-1)*2+j,(L-1)*2+j2,A_temp);
               
               gsl_matrix_complex_set(Bbig,(L-1)*2+j,(L-1)*2+j2,By_temp);
          }


       
       ///////////////////////create Full 2D Hamiltonian from blocks of coupled chains///////////////
         gsl_matrix_complex_set_zero(H);
       
         for(j=0;j<2*L;j++)
         for(j2=0;j2<2*L;j2++)
         {

            for(i=1;i<(L);i++)
            {

            A_temp = gsl_matrix_complex_get(Abig,j,j2);
            B_temp = gsl_matrix_complex_get(Bbig,j,j2);


                gsl_matrix_complex_set(H,(i-1)*2*(L)+j,(i-1)*2*(L)+j2, A_temp);

                gsl_matrix_complex_set(H,(i-1)*2*(L)+j2,i*2*(L)+j, gsl_complex_conjugate(B_temp));
                gsl_matrix_complex_set(H,i*2*(L)+j,(i-1)*2*(L)+j2, B_temp);
                

            }


               A_temp = gsl_matrix_complex_get(Abig,j,j2);
               B_temp = gsl_matrix_complex_get(Bbig,j,j2);

                gsl_matrix_complex_set(H,0+j2,(L-1)*2*(L)+j, gsl_complex_mul_real(gsl_complex_conjugate(B_temp),beta));
                gsl_matrix_complex_set(H,(L-1)*2*(L)+j,0+j2, gsl_complex_mul_real(B_temp,beta));
 
                gsl_matrix_complex_set(H,(L-1)*2*(L)+j,(L-1)*2*(L)+j2,A_temp);

        }


///////////////////////////////calculate Green's function //////////////////////////////
     
     int sig;//utility integer needed for matrix inversion routine
  
     /////////////Inverts "Hamiltonian" matrix to get the Green's function at each frequency
     gsl_linalg_complex_LU_decomp(H, p, &sig);
     gsl_linalg_complex_LU_invert (H, p, G);

 
        
             
        
        
        ///////////////////////////////////Calculates the 2D spectral function for each frequency/////////////////////
        int s1,s2;
        gsl_matrix_set_zero(spec);
        sfull=0.0;
        for(s1=0;s1<L;s1++)
        { for(s2=0;s2<2*L;s2++)
           {
            gsl_matrix_set(spec,(int)(s2/2),s1,GSL_IMAG(gsl_complex_add(gsl_matrix_complex_get(G,s2+2*L*s1,s2+2*L*s1),gsl_matrix_complex_get(G,s2+1+2*L*s1,s2+1+2*L*s1))));
            sfull+=gsl_matrix_get(spec,(int)(s2/2),s1);
            s2++;
           }
          
        }
          sfull=sfull/(PI*2*L*L);
      ///////////stores the spectral function in a file/////////////////////////
        f = fopen ("sfun.dat", "a+");
        fprint_mat(spec, L, L, f);
        fclose(f);
        
        f = fopen ("sfull.dat", "a+");
        fprintf(f,"%g\t %g\n",omega,sfull);
        fclose(f);
        
        
    
    }
////////////////////////////////End Processing, increase omega index by 1 and repeat//////////////////////////

   


////////////////////////////Clean Up////////////////////////////
//free all memory for allocated structures
    gsl_matrix_complex_free(G);
    gsl_permutation_free(p);
    gsl_matrix_complex_free(Bbig);
    gsl_matrix_complex_free(Abig);
    gsl_matrix_complex_free(A);
    gsl_matrix_free(spec);
    gsl_matrix_complex_free(Bx);
    gsl_matrix_complex_free(By);
    gsl_matrix_complex_free(H);
    gsl_matrix_complex_free(cmat_temp);
    gsl_matrix_free(omlist);
  

//default return value
    return 0;


}
