#include "util.h"



phys_para load_para()
{
    phys_para para = 
        {   50,             // size of the system
            1.0,        // hopping constant t
            3.0,        // d*g coupling constant V
            0.0,       // chemical potential   es
            1,0.0,0.0,0.0           // hopping stride of d
        };
        


    FILE * f = fopen("parameters.txt","r");
    if(f!=NULL)
    {
        fscanf(f, 
        "L=%d\n t=%g\n V=%g\n es=%g\n stride=%d\n exy=%g\n exz=%g\n eyz=%g",
        &(para.L), &(para.t), &(para.V), &(para.es),&(para.stride),&(para.exy),&(para.exz),&(para.eyz)
        );
    }
    else
    {
        printf("\n[Error]:\n\tFailed to load parameters from parameters.txt.");
        printf("\n\tDefaulat values are restored.\n\n");
    }
    fclose(f);
    
    
    
    return para;
}

int checkrecord(char * s)
{
    if(s[0]=='#')
        return -1;
    if(s[0]=='\r')
        return -1;
    if(s[0]=='\n')
        return -1;

    return 1;   
}

gsl_matrix * load_klistone()
{
    FILE * f = fopen("klist.txt", "r");
    
    float k1;
    char s[100];
    int size=0;

    gsl_matrix * ks;
    
    if(f!=NULL)
    {   
        while((fgets(s, 100,f))!=NULL)  
        {
            if((checkrecord(s))>0)
                size++;
        }
        fclose(f);
        
            
        if(size<1)
        {
            return NULL;
        }           
        
        f = fopen("klist.txt", "r");
        
        if(f==NULL)
        {
            printf("Failed to load k list.\n");
            return NULL;
        }
        
        ks=gsl_matrix_alloc(size,1);
        size=0;
        k1=0;
        
        while((fgets(s, 100,f))!=NULL)  
        {   
            
            if((checkrecord(s))>0)
            {   
                //k1++;     
                sscanf(s, "%f", &k1);
                
                gsl_matrix_set(ks,size,0,k1);
                
                size++; 
                
            }
        }
        
    }
    else
    {
        printf("Failed to load k list.\n");
        return NULL;
    
    }


//  printf("\n%d sets of k loaded.\n",size);    
    fclose(f);  
    
    return ks;
}






//------------------------------------------------------------------------------
//  add a to the element of m(i,j)
//------------------------------------------------------------------------------
void matrix_add_at(gsl_matrix * m, int i, int j, double a)
{
    double t = gsl_matrix_get(m, i, j);
    gsl_matrix_set(m,i,j, t+a);
}

void cmatrix_add_at(gsl_matrix_complex * m, int i, int j, gsl_complex a)
{
    gsl_complex t = gsl_matrix_complex_get(m, i, j);
    t = gsl_complex_add(t,a);
    gsl_matrix_complex_set(m,i,j, t);
}

//------------------------------------------------------------------------------
//  print real matrix
//------------------------------------------------------------------------------
void print_mat(gsl_matrix * data, int const n1, int const n2)
{
    int i, j;
        
    for(i=0; i<n1;i++)
    {
        printf("\n");
        for(j=0;j<n2;j++)
        {
    
            printf("%g   ", gsl_matrix_get(data,i,j));
        }
    }
}

//------------------------------------------------------------------------------
//  print real matrix to file
//------------------------------------------------------------------------------
void fprint_mat(gsl_matrix * data, int const n1, int const n2, FILE * f)
{
    int i, j;
        
    for(i=0; i<n1;i++)
    {
                       
        for(j=0;j<n2;j++)
        {   
            fprintf(f,"%g   ", gsl_matrix_get(data,i,j));
        }       
    fprintf(f,"\n"); 
    }
}

//------------------------------------------------------------------------------
//  print complex matrix
//------------------------------------------------------------------------------
void print_cmat(gsl_matrix_complex * data, int const n1, int const n2)
{
    int i, j;
    gsl_complex c_temp1;
    printf("\n------------------");
    for(i=0; i<n1;i++)
    {
        printf("\n");
        for(j=0;j<n2;j++)
        {
            c_temp1 = gsl_matrix_complex_get(data,i,j);
            printf("%g + I %g    \t", GSL_REAL(c_temp1), GSL_IMAG(c_temp1));
        }
    }
    printf("\n------------------\n");
}

//------------------------------------------------------------------------------
//  print complex matrix to file
//------------------------------------------------------------------------------
void fprint_cmat(gsl_matrix_complex * data, int const n1, int const n2, FILE *f)
{
    int i, j;
    gsl_complex c_temp1;
    
    for(i=0; i<n1;i++)
    {
        fprintf(f,"\n");                
        for(j=0;j<n2;j++)
        {
            c_temp1 = gsl_matrix_complex_get(data,i,j);
            fprintf(f, "%g + I %g  ", GSL_REAL(c_temp1), GSL_IMAG(c_temp1));
        }

    }
}

//------------------------------------------------------------------------------
//  print complex matrix to file
//------------------------------------------------------------------------------
void fprint_cmat2(gsl_matrix_complex * data, int const n1, int const n2, FILE *fre, FILE *fimg)
{
    int i, j;
    gsl_complex c_temp1;
    
    for(i=0; i<n1;i++)
    {
        fprintf(fre,"\n");
        fprintf(fimg,"\n");             
        for(j=0;j<n2;j++)
        {
            c_temp1 = gsl_matrix_complex_get(data,i,j);
            fprintf(fre, "%g ", GSL_REAL(c_temp1));
            fprintf(fimg, "%g ", GSL_IMAG(c_temp1));
        }

    }
}
