#ifndef _UTIL_H_
#define _UTIL_H_

#endif //_UTIL_H_

#define PI 3.1415926536

#include <stdio.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_complex.h>
#include <gsl/gsl_complex_math.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_fft_complex.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_eigen.h>


//A structure that will contain all of the parameters from the input file
typedef struct {
    int L;          // size of the system
    float t;        // boundary conditions along y
    float V;        // boundary conditions along x
    float es;       // eta
    int stride;     // extra, not used in this code
    float exy;      //spring constant along x
    float exz;      //spring constant along y
    float eyz;      //magnitude of magnetic field Bz 
  } phys_para ;
 

//------------------------------------------------------------------------------
//  Loads data from input file
//------------------------------------------------------------------------------
phys_para load_para(); 


//------------------------------------------------------------------------------
//  Make sure the file is the right one for input
//------------------------------------------------------------------------------
int checkrecord(char * s);

//------------------------------------------------------------------------------
//  Loads a set of frequencies (or wavevectors if that is what is needed)
//------------------------------------------------------------------------------
gsl_matrix * load_klistone();




//------------------------------------------------------------------------------
//  add a to the element of m(i,j)
//------------------------------------------------------------------------------
void matrix_add_at(gsl_matrix * m, int i, int j, double a);  
void cmatrix_add_at(gsl_matrix_complex * m, int i, int j, gsl_complex a);

//------------------------------------------------------------------------------
//  print real matrix
//------------------------------------------------------------------------------
void print_mat(gsl_matrix * data, int const n1, int const n2);



//------------------------------------------------------------------------------
//  print real matrix to file
//------------------------------------------------------------------------------
void fprint_mat(gsl_matrix * data, int const n1, int const n2, FILE * f);


//------------------------------------------------------------------------------
//  print complex matrix
//------------------------------------------------------------------------------
void print_cmat(gsl_matrix_complex * data, int const n1, int const n2);


//------------------------------------------------------------------------------
//  print complex matrix to file
//------------------------------------------------------------------------------
void fprint_cmat(gsl_matrix_complex * data, int const n1,int const n2,FILE * f);
void fprint_cmat2(gsl_matrix_complex * data, int const n1, int const n2, FILE *fre, FILE *fimg);

void gsl_matrix_complex_add_at(gsl_matrix_complex * g, int const i, int const j, gsl_complex a);


void fprint_vec(gsl_vector * data, int const n1,  FILE * f);

void check_hermitian(gsl_matrix_complex * data, int const n);
